<?php
/*
Plugin Name: WP htaccess Control
Plugin URI: http://dardna.com/wp-htaccess-control
Description: Interface to customize the permalinks (author, category, archives and pagination) and htaccess file generated by Wordpress.
Version: 2.1.2
Author: António Andrade
Author URI: http://dardna.com
*/
/*  Copyright 2010  António Andrade  (email : dardna@dardna.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
/*	Removal of the Category Base is based on WP No Category Base by Saurabh Gupta, 2008 */
if (!class_exists("WPhtc")) {
	class WPhtc {
		function WPhtc() { 
			}
		function wphtc_init(){
			# set locale
			$currentLocale = get_locale();
			if(!empty($currentLocale)) {
				$moFile = dirname(__FILE__) . "/lang/wp-htaccess-control-" . $currentLocale . ".mo";
				if(@file_exists($moFile) && is_readable($moFile)) load_textdomain('wp-htaccess-control', $moFile);
				}
			}
		function wphtc_check_first_run(){
			# flush rules to get some data filled
			$WPhtc_data=get_option('WPhtc_data');
			if(!isset($WPhtc_data['htaccess_original'])){
				$this->refresh_rewrite_rules();
				}
			}
		# Flush rules
		function refresh_rewrite_rules(){
				global $wp_rewrite;
				$wp_rewrite->flush_rules();
				//print_r($wp_rewrite);
				}
		# Filter pagination links generated by Wordpress
		function wphtc_filter_get_pagenum_link($result){
			$WPhtc_data=get_option('WPhtc_data');
			if(isset($WPhtc_data['cpp'])&&$WPhtc_data['cpp']!=''){
				$result=preg_replace("/".urlencode($WPhtc_data['cpp'])."\/[0-9]+[\/]?/","",$result);
				$result=str_replace("page",urlencode($WPhtc_data['cpp']),$result);
				}
			return $result;
			}
		# Filter link generated by get_author_posts_url() to use the Custom Author Permalink
		function wphtc_filter_author_link($link){
			$WPhtc_data=get_option('WPhtc_data');
			if(isset($WPhtc_data['cap']) && $WPhtc_data['cap']!=''){
				$link=str_replace("author",urlencode($WPhtc_data['cap']),$link);
				}
			return $link;
			}
		# Disable canonical redirection on urls using custom pagination permalink
		function wphtc_filter_redirect_canonical($redirect_url,$requested_url){
			$WPhtc_data=get_option('WPhtc_data');
			return false;
			if(isset($WPhtc_data['cpp'])&&$WPhtc_data['cpp']!=''&&get_query_var('paged') > 1&&strpos($requested_url,urlencode($WPhtc_data['cpp']))){
				return false;
				}
			else{
				return $redirect_url;
				}
			}
		# Rewrite Rulus: Add Category + Tag + Author Archives; Custom Pagination; Custom Author Base
		function wphtc_filter_rewrite_rules($rewrite_rules){
			$WPhtc_data=get_option('WPhtc_data');
			$page_base=($WPhtc_data['cpp']!='')?$WPhtc_data['cpp']:'page';
			# Category Archives
			if($WPhtc_data['category_archives']){
				# If category base removed
				if($WPhtc_data['remove_category_base']){
					$categories=get_categories(array('hide_empty'=>false));
					foreach($categories as $category) {
						$category_nicename = urldecode($category->slug);
						if ( $category->parent == $category->cat_ID ){
							$category->parent = 0;
							}
						elseif ($category->parent != 0 ){
							$category_nicename = urldecode(get_category_parents( $category->parent, false, '/', true )) . $category_nicename;
							}
						$new_rules = array( '('.$category_nicename.')/([0-9]{4})/?$' => 'index.php?category_name=$matches[1]&year=$matches[2]',
										'('.$category_nicename.')/([0-9]{4})/'.$page_base.'/?([0-9]{1,})/?$' => 'index.php?category_name=$matches[1]&year=$matches[2]&paged=$matches[3]',
										'('.$category_nicename.')/([0-9]{4})/([0-9]{2})/?$' => 'index.php?category_name=$matches[1]&year=$matches[2]&monthnum=$matches[3]',
										'('.$category_nicename.')/([0-9]{4})/([0-9]{2})/'.$page_base.'/?([0-9]{1,})/?$' => 'index.php?category_name=$matches[1]&year=$matches[2]&monthnum=$matches[3]&paged=$matches[4]');
						$rewrite_rules = $new_rules + $rewrite_rules;
						}
					}
				# else
				else {
					$category_base = (get_option('category_base')!='')?get_option('category_base'):'category';
					$new_rules = array( $category_base.'/([^/]+)/([0-9]{4})/?$' => 'index.php?category_name=$matches[1]&year=$matches[2]',
										$category_base.'/([^/]+)/([0-9]{4})/'.$page_base.'/?([0-9]{1,})/?$' => 'index.php?category_name=$matches[1]&year=$matches[2]&paged=$matches[3]',
										$category_base.'/([^/]+)/([0-9]{4})/([0-9]{2})/?$' => 'index.php?category_name=$matches[1]&year=$matches[2]&monthnum=$matches[3]',
										$category_base.'/([^/]+)/([0-9]{4})/([0-9]{2})/'.$page_base.'/?([0-9]{1,})/?$' => 'index.php?category_name=$matches[1]&year=$matches[2]&monthnum=$matches[3]&paged=$matches[4]');
					$rewrite_rules = $new_rules + $rewrite_rules;
					}
				}
			# Author Archives
			if(isset($WPhtc_data['author_archives'])){
				$author_base = ($WPhtc_data['cap']!='')?$WPhtc_data['cap']:'author';
				$new_rules = array( $author_base.'/([^/]+)/([0-9]{4})/?$' => 'index.php?author_name=$matches[1]&year=$matches[2]',
									$author_base.'/([^/]+)/([0-9]{4})/'.$page_base.'/?([0-9]{1,})/?$' => 'index.php?author_name=$matches[1]&year=$matches[2]&paged=$matches[3]',
									$author_base.'/([^/]+)/([0-9]{4})/([0-9]{2})/?$' => 'index.php?author_name=$matches[1]&year=$matches[2]&monthnum=$matches[3]',
									$author_base.'/([^/]+)/([0-9]{4})/([0-9]{2})/'.$page_base.'/?([0-9]{1,})/?$' => 'index.php?author_name=$matches[1]&year=$matches[2]&monthnum=$matches[3]&paged=$matches[4]');
				$rewrite_rules = $new_rules + $rewrite_rules;
				}
			# Tag Archives
			if(isset($WPhtc_data['tag_archives'])){
				$new_rules = array( 'tag/([^/]+)/([0-9]{4})/?$' => 'index.php?tag=$matches[1]&year=$matches[2]',
									'tag/([^/]+)/([0-9]{4})/'.$page_base.'/?([0-9]{1,})/?$' => 'index.php?tag=$matches[1]&year=$matches[2]&paged=$matches[3]',
									'tag/([^/]+)/([0-9]{4})/([0-9]{2})/?$' => 'index.php?tag=$matches[1]&year=$matches[2]&monthnum=$matches[3]',
									'tag/([^/]+)/([0-9]{4})/([0-9]{2})/'.$page_base.'/?([0-9]{1,})/?$' => 'index.php?tag=$matches[1]&year=$matches[2]&monthnum=$matches[3]&paged=$matches[4]');
				$rewrite_rules = $new_rules + $rewrite_rules;
				}
			# Custom Pagination; Custom Author Permalink
			if((isset($WPhtc_data['cap'])&&$WPhtc_data['cap']!='')||(isset($WPhtc_data['cpp'])&&$WPhtc_data['cpp']!='')){
				$rewrite_rules=serialize($rewrite_rules);
				# Author
				if(isset($WPhtc_data['cap'])&&$WPhtc_data['cap']!=''){
					$rewrite_rules=str_replace('author/',$WPhtc_data['cap'].'/',$rewrite_rules);
					}
				# Pagination
				if(isset($WPhtc_data['cpp'])&&$WPhtc_data['cpp']!=''){
					$rewrite_rules=str_replace('page/',$WPhtc_data['cpp'].'/',$rewrite_rules);
					}
				$rewrite_rules=unserialize(preg_replace('!s:(\d+):"(.*?)";!se', "'s:'.strlen('$2').':\"$2\";'",$rewrite_rules));
				}
			return $rewrite_rules;
			}
		# Adding our Custom Author Base to the sitemap generated by Google XML Sitemaps
		# Adapted from sitemap-core.php (line ~2036)
		function set_sm(){
			$WPhtc_data=get_option('WPhtc_data');
			if(class_exists('GoogleSitemapGenerator')&&$WPhtc_data['cap']!=''){
				$generatorObject = &GoogleSitemapGenerator::GetInstance();
				if ($generatorObject != null){
					global $wpdb;
					$sql = "SELECT DISTINCT
							p.ID,
							u.user_nicename,
							MAX(p.post_modified_gmt) AS last_post
						FROM
							{$wpdb->users} u,
							{$wpdb->posts} p
						WHERE
							p.post_author = u.ID
							AND p.post_status = 'publish'
							AND p.post_type = 'post'
							AND p.post_password = ''
							" . (floatval($wp_version) < 2.1?"AND p.post_date_gmt <= '" . gmdate('Y-m-d H:i:59') . "'":"") . "
						GROUP BY
							u.ID,
							u.user_nicename";
					$authors = $wpdb->get_results($sql);	
					if($authors && is_array($authors)) {
						foreach($authors as $author) {
							$url=get_bloginfo('home')."/".$WPhtc_data['cap']."/".$author->user_nicename."/";
							$generatorObject->AddUrl($url,$generatorObject->GetTimestampFromMySql($author->last_post),$generatorObject->GetOption("cf_auth"),$generatorObject->GetOption("pr_auth"));
							}
						}
					}
				}
			}
		# Adding rewrite pairs to the .htaccess generated by Wordpress, everytime the rules are flush
		function wphtc_wp_rewrite_rules($wp_rewrite){
			$WPhtc_data=get_option('WPhtc_data');
			
				# Deprecate in the future
				for($i=0;$i<count($WPhtc_data['pats']);$i++){
					if($WPhtc_data['pats'][$i]!=''&&$WPhtc_data['subs'][$i]!=''){
						# Adding each custom rewrite pair to $wp_rewrite->non_wp_rules['pattern']='substitution'
						$wp_rewrite->non_wp_rules[$WPhtc_data['pats'][$i]]=$WPhtc_data['subs'][$i];
						}
					}
				
			# Keeping a copy of the generated htaccess in our option array for later reviewing
			$WPhtc_data['cur_hta']='# BEGIN Wordpress<br/>'.nl2br($wp_rewrite->mod_rewrite_rules())."# END Wordpress";
				# deprecating original_hta
				unset($WPhtc_data['original_hta']);
			update_option('WPhtc_data',$WPhtc_data);
			return $wp_rewrite;
			}
		# Adding Custom Rules to htaccess file generated by Wordpress, everytime the rules are flush
		function wphtc_hta_rules($rules){
			$WPhtc_data=get_option('WPhtc_data');
			# keeping original hta
			if(!isset($WPhtc_data['htaccess_original'])){
				$WPhtc_data['htaccess_original']=$rules;
				update_option('WPhtc_data',$WPhtc_data);
				}
			# Maintenance Mode
			if($WPhtc_data['maintenance_active']&&isset($WPhtc_data['maintenance_ips'][0])){
				$no_ips=count($WPhtc_data['maintenance_ips']);
				$new_rules.="\n# WPhtc: Begin Maintenance Mode\n";
				# redirect
				if($WPhtc_data['maintenance_redirection']!=''){
					$new_rules.="RewriteEngine on\n";
					$new_rules.="RewriteCond %{REQUEST_URI} !\.(jpe?g|png|gif) [NC]\n";
					for($i=0;$i<$no_ips;$i++){
						$new_rules.="RewriteCond %{REMOTE_HOST} !^".str_replace(".","\.",trim($WPhtc_data['maintenance_ips'][$i]))."\n";
						}
					if(substr($WPhtc_data['maintenance_redirection'],0,1)=="/"){
						$new_rules.="RewriteCond %{REQUEST_URI} !".$WPhtc_data['maintenance_redirection']."$ [NC]\n";
						}
					$new_rules.="RewriteRule .* ".$WPhtc_data['maintenance_redirection']." [R=302,L]\n";
					}
				# no redirection
				else{
					$new_rules.="order deny,allow\n";
					$new_rules.="deny from all\n";
					foreach($WPhtc_data['maintenance_ips'] as $ip){
						$new_rules.="allow from ".$ip."\n";
						}
					}
				$new_rules.="# WPhtc: End Maintenance Mode\n";
				}
			# Custom htaccess
			if($WPhtc_data['hta']){
				$new_rules.="\n# WPhtc: Begin Custom htaccess\n";
				$new_rules.=stripslashes($WPhtc_data['hta'])."\n";
				$new_rules.="# WPhtc: End Custom htaccess\n";
				}
			# htaccess suggestions
			if($WPhtc_data['disable_serversignature']){
				$new_rules.="\n# WPhtC: Disable ServerSignature on generated error pages\n";
				$new_rules.="ServerSignature Off\n";
				}
			if($WPhtc_data['admin_email']){
				$new_rules.="\n# WPhtC: Set admin email\n";
				$new_rules.="SetEnv SERVER_ADMIN ".$WPhtc_data['admin_email']."\n\n";
				}
			if($WPhtc_data['disable_indexes']){
				$new_rules.="\n# WPhtC: Disable directory browsing\n";
				$new_rules.="Options All -Indexes\n";
				}
			if($WPhtc_data['up_limit']){
				$new_rules.="\n# WPhtC: Limit upload size to ".$WPhtc_data['up_limit']." MB\n";
				$new_rules.="LimitRequestBody ".($WPhtc_data['up_limit']*1024*1024)." \n";
				}
			if($WPhtc_data['redirect_500']){
				$new_rules.="\n# WPhtC: Setting 500 Error page\n";
				$new_rules.="ErrorDocument 500 ".$WPhtc_data['redirect_500']."\n";
				}
			if($WPhtc_data['redirect_403']){
				$new_rules.="\n# WPhtC: Setting 403 Error page\n";
				$new_rules.="ErrorDocument 403 ".$WPhtc_data['redirect_403']."\n";
				}
			if($WPhtc_data['protect_wp_config']){
				$new_rules.="\n# WPhtC: Protect WP-config.php\n";
				$new_rules.="<files wp-config.php>\n";
				$new_rules.="order allow,deny\n";
				$new_rules.="deny from all\n";
				$new_rules.="</files>\n";
				}
			if($WPhtc_data['protect_htaccess']){
				$new_rules.="\n# WPhtC: Protect .htaccess file\n";
				$new_rules.="<files ~ \"^.*\.([Hh][Tt][Aa])\">\n";
				$new_rules.="order allow,deny\n";
				$new_rules.="deny from all\n";
				$new_rules.="</files>\n";
				}
			if($WPhtc_data['disable_hotlink']){
				$new_rules.="\n# WPhtC: Disable image hotlinking\n";
				$new_rules.="<IfModule mod_rewrite.c>\n";
				$new_rules.="RewriteEngine On\n";
				$new_rules.="RewriteCond %{HTTP_REFERER} !^$\n";
				$new_rules.="RewriteCond %{HTTP_REFERER} !^http://(www\.)?".str_ireplace(array("http://","www."),"",get_bloginfo("url"))."/.*$ [NC]\n";
				$new_rules.="RewriteRule \.(jpg|jpeg|png|gif)$ ".$WPhtc_data['disable_hotlink']." [NC,R,L]\n";
				$new_rules.="</IfModule>\n";
				}
			if($WPhtc_data['canon']){
				$url=str_ireplace(array("http://","www."),"",get_bloginfo("url"));
				$domain=explode("/",$url);
				$escaped_domain=str_ireplace(".","\.",$domain[0]);
				if($WPhtc_data['canon']=='www'){
					$siteurl=get_option('siteurl');
					if (!preg_match('/^http:\/\/www\./', $siteurl)) {
						$siteurl=str_replace("http://","http://www.",$siteurl);
						update_option('siteurl',$siteurl);
						}
					$homeurl=get_option('home');
					if (!preg_match('/^http:\/\/www\./', $homeurl)) {
						$homeurl=str_replace("http://","http://www.",$homeurl);
						update_option('home',$homeurl);
						}
					}
				else if($WPhtc_data['canon']=='simple'){
					$siteurl=get_option('siteurl');
					if (preg_match('/^http:\/\/www\./', $siteurl)) {
						$siteurl=str_replace("http://www.","http://",$siteurl);
						update_option('siteurl',$siteurl);
						}
					$homeurl=get_option('home');
					if (preg_match('/^http:\/\/www\./', $homeurl)) {
						$homeurl=str_replace("http://www.","http://",$homeurl);
						update_option('home',$homeurl);
						}
					}
				}
			if($WPhtc_data['gzip']){
					$new_rules.="\n# WPhtC: Setting mod_gzip\n";
					$new_rules.="<ifModule mod_gzip.c>\n";
					$new_rules.="mod_gzip_on Yes\n";
					$new_rules.="mod_gzip_dechunk Yes\n";
					$new_rules.="mod_gzip_item_include file \.(html?|txt|css|js|php|pl)$\n";
					$new_rules.="mod_gzip_item_include handler ^cgi-script$\n";
					$new_rules.="mod_gzip_item_include mime ^text/.*\n";
					$new_rules.="mod_gzip_item_include mime ^application/x-javascript.*\n";
					$new_rules.="mod_gzip_item_exclude mime ^image/.*\n";
					$new_rules.="mod_gzip_item_exclude rspheader ^Content-Encoding:.*gzip.*\n";
					$new_rules.="</ifModule>\n";
				}
			if($WPhtc_data['deflate']){
					$new_rules.="\n# WPhtC: Setting mod_deflate\n";
					$new_rules.="<IfModule mod_deflate.c>\n";
					$new_rules.="AddOutputFilterByType DEFLATE text/html text/plain text/xml application/xml application/xhtml+xml text/javascript text/css application/x-javascript\n";	
					$new_rules.="BrowserMatch ^Mozilla/4 gzip-only-text/html\n";
					$new_rules.="BrowserMatch ^Mozilla/4.0[678] no-gzip\n";
					$new_rules.="BrowserMatch bMSIE !no-gzip !gzip-only-text/html\n";
					$new_rules.="Header append Vary User-Agent env=!dont-vary\n";
					$new_rules.="</IfModule>\n";
				}
			if($WPhtc_data['wp_hta']&&trim($WPhtc_data['wp_hta'])!=''){
				$new_rules.="\n".$WPhtc_data['wp_hta'];
				}
			else{
				$new_rules.="\n".$rules;
				}
			return $new_rules;
			}
		# Category Base	
		function filter_category_link($catlink, $category_id) {
			$WPhtc_data=get_option('WPhtc_data');
			if($WPhtc_data['remove_category_base']){
				$category = &get_category( $category_id );
				if ( is_wp_error( $category ) ){
					return $category;
					}
				$category_nicename = $category->slug;
				if ( $category->parent == $category_id ){ 
					$category->parent = 0;
					}
				elseif ($category->parent != 0 ){
					$category_nicename = get_category_parents( $category->parent, false, '/', true ) . $category_nicename;
					}
				$catlink = trailingslashit(get_option( 'home' )) . user_trailingslashit( $category_nicename, 'category' );
				}
			return $catlink;
			}
		# Filter Category Rewrite Rules
		function remove_category_base_from_rewrite_rules($category_rewrite) {
			$WPhtc_data=get_option('WPhtc_data');
			if($WPhtc_data['remove_category_base']){
				$category_rewrite=array();
				$categories=get_categories(array('hide_empty'=>false));
				foreach($categories as $category) {
					$category_nicename = urldecode($category->slug);
					if ( $category->parent == $category->cat_ID ){
						$category->parent = 0;
						}
					elseif ($category->parent != 0 ){
						$category_nicename = urldecode(get_category_parents( $category->parent, false, '/', true )) . $category_nicename;
						}
					$category_rewrite['('.$category_nicename.')/(?:feed/)?(feed|rdf|rss|rss2|atom)/?$'] = 'index.php?category_name=$matches[1]&feed=$matches[2]';
					$category_rewrite['('.$category_nicename.')/page/?([0-9]{1,})/?$'] = 'index.php?category_name=$matches[1]&paged=$matches[2]';
					$category_rewrite['('.$category_nicename.')/?$'] = 'index.php?category_name=$matches[1]';
					}
				}
			return $category_rewrite;
			}
		# WP-htaccess Control options page interface
		function wphtc_page(){
			include (dirname (__FILE__).'/wp-htaccess-control-ui.php');
			}
		# Adding our options page to the admin menu
		function configure_menu(){
			if(current_user_can("administrator")){
				$page=add_submenu_page("options-general.php","WP htaccess Control", "WP htaControl", 6, __FILE__, array('WPhtc','wphtc_page'));
				add_action('admin_print_scripts-'.$page, array('WPhtc','wphtc_page_script'));
				add_action('admin_print_styles-'.$page, array('WPhtc','wphtc_page_style'));
				}
			}
		# Enqueue Ui Scripts on Plugin page
		function wphtc_page_script(){
			wp_enqueue_script("htaccess-control-js", WP_PLUGIN_URL . '/wp-htaccess-control/wp-htaccess-control-ui.js');  
			}
		# Enqueue Ui Scripts on Plugin page
		function wphtc_page_style(){
			wp_enqueue_style("htaccess-control-css", WP_PLUGIN_URL . '/wp-htaccess-control/wp-htaccess-control-ui.css');  
			}
		# Options Page Actions
		function wphtc_page_action(){
			WPhtc::wphtc_check_first_run();
			$action=$_POST['action']?$_POST['action']:$_GET['action'];
			global $echo;
			if(isset($action)){
				$WPhtc_data=get_option('WPhtc_data');
				switch($action){
					# if reseting everything just delete the option array
					case 'reset_rules':
						delete_option('WPhtc_data');
						$echo.=__('All rules reset.', 'wp-htaccess-control');
						$this->refresh_rewrite_rules();
						break;
					# if updating, save new rules in database and flush rewrite rules
					case 'update':	
						# get Rewrite Pairs
						for($i=0;$i<$_POST['WPhtc_num'];$i++){ 
							if(!empty($_POST['WPhtc_pat'.$i]) && !empty($_POST['WPhtc_sub'.$i])){
								$WPhtc_data['pats'][$i]=$_POST['WPhtc_pat'.$i];								
								$WPhtc_data['subs'][$i]=$_POST['WPhtc_sub'.$i];
								}					
							}
						if(count($WPhtc_data['pats'])>1 && $WPhtc_data['pats'][0]==''){
							unset($WPhtc_data['pats'][0],$WPhtc_data['subs'][0]);
							}
						if(is_array($WPhtc_data['pats'])&&is_array($WPhtc_data['subs'])){
							$WPhtc_data['pats']=array_values($WPhtc_data['pats']);
							$WPhtc_data['subs']=array_values($WPhtc_data['subs']);
							}
						# get Custom Htaccess
						$WPhtc_data['hta']=$_POST['WPhtc_hta'];
						# get Custom Author Permalink
						$WPhtc_data['cap']=$_POST['WPhtc_cap'];
						# get Custom Pagination Permalink
						$WPhtc_data['cpp']=$_POST['WPhtc_cpp'];
						# wordpress htaccess and jim morgan's htaccess
						$WPhtc_data['wp_hta']=$_POST['WPhtc_wp_hta']."\n";
						$WPhtc_data['jim_morgan_hta']=$_POST['WPhtc_jim_morgan_hta'];
						if($WPhtc_data['jim_morgan_hta']){
							$WPhtc_data['wp_hta'] ="\nRewriteEngine on\n";
							$WPhtc_data['wp_hta'].="# Unless you have set a different RewriteBase preceding this point,\n";
							$WPhtc_data['wp_hta'].="# you may delete or comment-out the following RewriteBase directive:\n";
							$WPhtc_data['wp_hta'].="RewriteBase /\n";
							$WPhtc_data['wp_hta'].="# if this request is for \"/\" or has already been rewritten to WP\n";
							$WPhtc_data['wp_hta'].="RewriteCond $1 ^(index\.php)?$ [OR]\n";
							$WPhtc_data['wp_hta'].="# or if request is for image, css, or js file\n";
							$WPhtc_data['wp_hta'].="RewriteCond $1 \.(gif|jpg|jpeg|png|css|js|ico)$ [NC,OR]\n";
							$WPhtc_data['wp_hta'].="# or if URL resolves to existing file\n";
							$WPhtc_data['wp_hta'].="RewriteCond %{REQUEST_FILENAME} -f [OR]\n";
							$WPhtc_data['wp_hta'].="# or if URL resolves to existing directory\n";
							$WPhtc_data['wp_hta'].="RewriteCond %{REQUEST_FILENAME} -d\n";
							$WPhtc_data['wp_hta'].="# then skip the rewrite to WP\n";
							$WPhtc_data['wp_hta'].="RewriteRule ^(.*)$ - [S=1]\n";
							$WPhtc_data['wp_hta'].="# else rewrite the request to WP\n";
							$WPhtc_data['wp_hta'].="RewriteRule . /index.php [L]\n";
							}
						# Remove Category Base
						$WPhtc_data['remove_category_base']=$_POST['WPhtc_remove_category_base'];
						# Advanced Archives
						$WPhtc_data['category_archives']=$_POST['WPhtc_category_archives'];
						$WPhtc_data['author_archives']=$_POST['WPhtc_author_archives'];
						$WPhtc_data['tag_archives']=$_POST['WPhtc_tag_archives'];
						# get htaccess Suggestions
						$WPhtc_data['disable_serversignature']=$_POST['WPhtc_disable_serversignature'];
						$WPhtc_data['disable_indexes']=$_POST['WPhtc_disable_indexes'];
						$WPhtc_data['up_limit']=(is_numeric($_POST['WPhtc_up_limit'])&&$_POST['WPhtc_up_limit']>0)?$_POST['WPhtc_up_limit']:'';
						$WPhtc_data['protect_wp_config']=$_POST['WPhtc_protect_wp_config'];
						$WPhtc_data['protect_htaccess']=$_POST['WPhtc_protect_htaccess'];
						$WPhtc_data['disable_hotlink']=trim($_POST['WPhtc_disable_hotlink']);
						$WPhtc_data['redirect_500']=trim($_POST['WPhtc_redirect_500']);
						$WPhtc_data['redirect_403']=trim($_POST['WPhtc_redirect_403']);
						$WPhtc_data['canon']=$_POST['WPhtc_canon'];
						$WPhtc_data['admin_email']=trim($_POST['WPhtc_admin_email']);
						$WPhtc_data['deflate']=trim($_POST['WPhtc_deflate']);
						$WPhtc_data['gzip']=trim($_POST['WPhtc_gzip']);
						# get maintenance
						$WPhtc_data['maintenance_active']=$_POST['WPhtc_maintenance_active'];
						$lines=preg_split("/\n|,/",$_POST['WPhtc_maintenance_ips']);
						$WPhtc_data['maintenance_ips']=array();
						foreach($lines as $line){
							trim($line);
							if(preg_match("/[a-z,0-9,\.]/",$line)){
								$WPhtc_data['maintenance_ips'][]=$line;
								}
							}
						$WPhtc_data['maintenance_redirection']=trim($_POST['WPhtc_maintenance_redirection']);
						
						# Unsetting inclusion of Author pages on Google XML Sitemap options, we'll add those links to the sitemap later with our Custom Author Permalink
						$WPhtc_data['sm_enabled']=$_POST['WPhtc_sm_enabled'];
						if($WPhtc_data['sm_enabled']==true){
							$SMoptions=get_option("sm_options");
							if($SMoptions&&is_array($SMoptions)){
								$SMoptions=get_option("sm_options");
								$SMoptions['sm_in_auth']=0;
								update_option('sm_options',$SMoptions);
								# Try to rebuild Sitemap
								do_action("sm_rebuild");
								}
							}
						# Update WP htaccess Control options
						update_option('WPhtc_data',$WPhtc_data);
						# Flush Rewrite Rules
						$this->refresh_rewrite_rules();
						$echo.=__('All options updated.', 'wp-htaccess-control');
						break;
					# deleting individual rewrite pair
					case 'delete_pair':
						if(isset($_GET['pair'])){
							# if it's the last pair
							if($_GET['pair']==0 && !isset($WPhtc_data['pats'][1])){
								unset($WPhtc_data['pats'], $WPhtc_data['subs']);
								$echo.=__('Wordpress Rewrite pairs reset.', 'wp-htaccess-control');
								}
							else{
								unset($WPhtc_data['pats'][$_GET['pair']],$WPhtc_data['subs'][$_GET['pair']]);
								$WPhtc_data['pats']=array_values($WPhtc_data['pats']);
								$WPhtc_data['subs']=array_values($WPhtc_data['subs']);
								$echo.=__('Wordpress Rewrite pair deleted.', 'wp-htaccess-control');
								}
							update_option('WPhtc_data',$WPhtc_data);
							$this->refresh_rewrite_rules();
							}
						break;
					}
				}
			}
		}
	} 
if (class_exists("WPhtc")) {
	$WPhtc = new WPhtc();
	}
if (isset($WPhtc)) {
	add_filter('mod_rewrite_rules', array($WPhtc,'wphtc_hta_rules'));
	add_filter('generate_rewrite_rules', array($WPhtc,'wphtc_wp_rewrite_rules'));
	add_filter('rewrite_rules_array',array($WPhtc,'wphtc_filter_rewrite_rules'));
	add_filter('author_link',array($WPhtc,'wphtc_filter_author_link'));
	add_filter('get_pagenum_link',array($WPhtc,'wphtc_filter_get_pagenum_link'));
	add_filter('redirect_canonical',array($WPhtc,'wphtc_filter_redirect_canonical'),10,4);
	add_action('admin_menu', array($WPhtc,'configure_menu'));
	add_action('sm_buildmap',array($WPhtc,'set_sm'));
	add_action('init', array($WPhtc,'wphtc_init'));
	add_filter('category_rewrite_rules', array($WPhtc,'remove_category_base_from_rewrite_rules'));
	add_filter('category_link', array($WPhtc,'filter_category_link'),1000,2);
	add_action('created_category',array($WPhtc,'refresh_rewrite_rules'));
	add_action('edited_category',array($WPhtc,'refresh_rewrite_rules'));
	add_action('delete_category',array($WPhtc,'refresh_rewrite_rules'));
	}
?>
